// Author : Quentin BENETHUILLERE
// Date of creation : 2021/03/22
// Last modification : 2021/03/22

// ------------------
// CLOCK
// ------------------
#define F_CPU 8000000

// ------------------
// LIBRARIES
// ------------------
#include <avr/io.h>
#include <util/delay.h>

// ------------------
// DEFINE
// ------------------
#define PIN_MOSFET 3 // Input pin connected to the push button.
#define PIN_BUTTON 4 // Output pin connected to the MOSFET gate.
#define T_OPEN 3000 // Time (in ms) during which the lock will remain open (or LED will remain on) after a button push.
#define T_COOL_DOWN 5000 // Time (in ms) during which the it will not be possible to open the lock or to turn ON the LED after a button push. This is a protection to let the solenoid cool down. Normally it should not be powered more than 10 or 15 seconds.     

/*
ATTiny45
*/
// Button = PB4 = INPUT. 0 when button is pushed, 1 when button is released.
// MOSFEFT gate = PB3 = OUTPUT.

int main(void)
{
	DDRB = 0b11101111; // Defining input/output pins (0=input)
	PORTB = PORTB & ~(1<<PIN_MOSFET); // MOSFET base = 0
	
	while(1)
		{
		
	  	if ((PINB & (1<<PIN_BUTTON)) == 0) // Button pushed.
	  	{
				PORTB = PORTB | (1<<PIN_MOSFET) ; // MOSFET base = 1
				_delay_ms(T_OPEN);
				PORTB = PORTB & ~(1<<PIN_MOSFET); // MOSFET base = 0
				_delay_ms(T_COOL_DOWN);				
			}
		else
		{
			PORTB = PORTB & ~(1<<PIN_MOSFET); // MOSFET base = 0
		}
	}
	return 0; // To avoid warning messages from the compiler (although the program will never reach this line in reality)
}
