// Author : Quentin BENETHUILLERE
// Date of creation : 2021/03/22
// Last modification : 2021/03/22

/*
 * This program aims at controlling a lock mechanism or a LED with a push button.
 * The idea is not that the lock mechanism is open as long as the button is pushed, but rather that the lock mechanism remains open for a certain time following the button push action, and then closes for a minimum time (to let the solenoid cool down).
 */

// -------------------------
// DECLARATION OF DEFINES 
// -------------------------

#define PIN_BUTTON 4 // Input pin connected to the push button.
#define PIN_MOSFET 3 // Output pin connected to the MOSFET gate.
#define T_OPEN 3000 // Time (in ms) during which the lock will remain open (or LED will remain on) after a button push.
#define T_COOL_DOWN 5000 // Time (in ms) during which the it will not be possible to open the lock or to turn ON the LED after a button push. This is a protection to let the solenoid cool down. Normally it should not be powered more than 10 or 15 seconds.

// -------------------------
// DECLARATION OF VARIABLES 
// -------------------------

// ------------------------------------------------
// Adjustable variables (program main parameters)
// ------------------------------------------------

// ------------------------------------------------
// PIN connections
// ------------------------------------------------

// ------------------------------------------------
// Other variables
// ------------------------------------------------
char button_state = 1; // Button not pushed

// -------------------------
// INITIALIZATION 
// -------------------------

// Setup code. Run once at power up, or each time the RESET button is pressed:
void setup() {

// ------------------------------------------------
// Input / Output
// ------------------------------------------------

  pinMode(PIN_BUTTON, INPUT);   // pin connected to the push button = input.  
  pinMode(PIN_MOSFET, OUTPUT);   // pin connected to the MOSFET gate = output.

// ------------------------------------------------
// PIN initial states
// ------------------------------------------------
  digitalWrite(PIN_BUTTON, HIGH); // HIGH state = button not pushed.
  digitalWrite(PIN_MOSFET, LOW);

} // end of setup() function

// -------------------------
// MAIN
// -------------------------

// Loop function that runs repeatedly:
void loop() {
  button_state = digitalRead(PIN_BUTTON);
  if (button_state==0){ // button pushed
    digitalWrite(PIN_MOSFET, HIGH);    // open the lock mechanism
    delay(T_OPEN);
    digitalWrite(PIN_MOSFET, LOW);    // close the lock mechanism
    delay(T_COOL_DOWN);
  }
  else{ // button NOT pushed
    digitalWrite(PIN_MOSFET, LOW);    // close the lock mechanism
  }

} // end of loop() function
